﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2011.06.29.
 * Idő: 12:25
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.IO;

namespace TetheredSun 
{
	/// <summary>
	/// Description of MultipleExtensionFileName.
	/// </summary>
	public class MultipleExtensionFileName
	{
		protected string fullPath;
		protected string directoryName;
		protected string fileNameWithoutExtension;
		protected string[] extensions;
		
		public MultipleExtensionFileName(string fullPath)
		{
			this.fullPath = fullPath;
			extensions = GetExtensions(fullPath, out directoryName, out fileNameWithoutExtension);
		}
		
		
		public string DirectoryName {
			get { return directoryName; }
			set { directoryName = value; }
		}
		
		public string[] Extensions {
			get { return extensions; }
			set { extensions = value; }
		}
		
		public string FileName {
			get {
				BuildFileName();
				return Path.GetFileName(fullPath);
			}
		}
		
		

		public string FileNameWithoutExtension {
			get { return fileNameWithoutExtension; }
			set { fileNameWithoutExtension = value; }
		}
		
		public string FullPath {
			get {
				BuildFileName();
				return fullPath;
			}
		}
		
		public string this[int index] {
			get { return extensions[index]; }
			set { extensions[index] = value; }
		}
		
		
		protected void BuildFileName()
		{
			string extensionString = String.Empty;
			
			if (extensions != null) {
				for (int i = extensions.Length - 1; i >= 0; i--) {
					extensionString += "." + extensions[i];
				}
			}
			
			fullPath = Path.Combine(directoryName, fileNameWithoutExtension + extensionString);
		}
		
		public bool ContainsExtension(params string[] extensions)
		{
			for (int i = 0; i < extensions.Length; i++) {
				if (Array.IndexOf<string>(this.extensions, extensions[i]) < 0) return false;
			}
			
			return true;
		}
		
		/// Gets the extension list, indexed from last to first, of a file name.
		/// </summary>
		/// <param name="fullPath">The full path to the file whose name is to be analysed.</param>
		/// <param name="directoryName">The name of the directory containing the file.</param>
		/// <param name="fileNameWithoutExtension">The name of the file, stripped of all extensions.</param>
		/// <returns></returns>
		public static string[] GetExtensions(string fullPath, out string directoryName, out string fileNameWithoutExtension)
		{
			string[] extensions;
			string[] split;
			string namePart = Path.GetFileName(fullPath);
			
			directoryName = Path.GetDirectoryName(fullPath);
			
			if (String.IsNullOrEmpty(namePart)) {
				fileNameWithoutExtension = null;
				return null;
			}
			
			split = namePart.Split(new string[] { "." }, StringSplitOptions.RemoveEmptyEntries);
			extensions = (split.Length > 0) ? new string[split.Length - 1] : null;
			
			if (extensions == null) {
				fileNameWithoutExtension = null;
				return null;
			}
			
			for (int i = 0; i < extensions.Length; i++) {
				extensions[i] = split[split.Length - 1 - i];
			}
			
			fileNameWithoutExtension = split[0];
			
			return extensions;
		}
		
		public static implicit operator string(MultipleExtensionFileName fileName)
		{
			return fileName.FullPath;
		}
		
		public static implicit operator MultipleExtensionFileName(string fileName)
		{
			return new MultipleExtensionFileName(fileName);
		}
	
		public override string ToString()
		{
			return FullPath;
		}
		
	}
}
