﻿/*
 * Created by SharpDevelop.
 * User: phil
 * Date: 2010.03.30.
 * Time: 15:01
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Text;
using System.Windows.Forms;
//
using TetheredSun;
using TetheredSun.GnuPlot;

namespace TetheredSun.Controls
{
	public class NoiseGraph : Control
	{
		protected Font				scaleFont = new Font("Trebuchet MS", 12.0f);
		protected Pen				axisPen = new Pen(Color.Black, 1.0f);
		protected Pen				tickPen = new Pen(Color.Black, 1.0f);
		protected Pen				gridPen = new Pen(Color.Gray, 1.0f);
		protected Pen				zoomPen;
		protected Brush				scaleBrush = new SolidBrush(Color.Black);
		protected Brush				zoomBrush;
		//
		protected Axis				axisX = new Axis(AxisType.X);
		protected Axis				axisY = new Axis(AxisType.Y);
		//
		protected Rectangle			drawRectangle;
		protected Rectangle			axisFrameRectangle;
		protected Rectangle 		curveRectangle;
		protected Rectangle			scaledAxisFrameRectangle;
		protected Rectangle			xAxisRectangle;
		protected Rectangle			yAxisRectangle;
		protected Rectangle			titleRectangle;
		protected RectangleF		labelRectangle;
		protected double			xMax;
		protected double			xMin;
		protected double			yMin;
		protected double			yMax;
		protected double			x0;
		protected double			y0;
		protected double			dx;
		protected double			dy;
		protected int				numberOfPoints;
		protected int				scalePadding = 5;
		protected int				xGridCount = -1;
		protected int				yGridCount = -1;
		protected bool				enforceTickBounds = true;
		protected bool				boundsUndefined = true;
		protected bool				controlPressed = false;
		protected bool				scrollingEnabled = true;
		protected bool				zoomingEnabled = true;
		protected bool				showCursorPosition = false;
		protected bool				checkRangeOnPainting = true;
		protected CurveList			curves = new CurveList();
		protected SmoothingMode 	lineSmoothingMode = SmoothingMode.Default;
		protected SmoothingMode 	symbolSmoothingMode = SmoothingMode.Default;
		protected TextRenderingHint	textRenderingHint = TextRenderingHint.SystemDefault;
		protected Point<double>		cursorCoordinates = null;
		protected GraphicsState		state;
		protected ContextMenuStrip 	contextMenu;
		protected Point?			mouseDownPoint = null;
		protected Point?			mouseUpPoint = null;
		protected Rectangle?		zoomRectangle = null;
		protected Rectangle?		previousZoomRectangle = null;
		protected Rectangle?		zoomInvalidateRectangle = null;
		protected Color				zoomColour;
		protected string[]			contextMenuLabels = new string[] { "Reset zoom", "Enable scrolling", "Enable zooming", "Save as GnuPlot script...", "Save as image..." };
		
		public NoiseGraph() : base()
		{
			this.Padding = new Padding(20);
			this.DoubleBuffered = true;
			SetStyle(ControlStyles.SupportsTransparentBackColor, true);
			//
			axisX.Title.Text = "X axis";
			axisY.Title.Text = "Y axis";
			//
			gridPen.DashStyle = DashStyle.Dot;
			gridPen.DashCap = DashCap.Round;
			//
			zoomPen = new Pen(Color.CornflowerBlue, 1.0f);
			zoomColour = Color.FromArgb(100, zoomPen.Color.R, zoomPen.Color.G, zoomPen.Color.B);
			zoomBrush = new SolidBrush(zoomColour);
			contextMenu = new ContextMenuStrip();
			foreach (string label in contextMenuLabels) {
				contextMenu.Items.Add(new ToolStripMenuItem(label, null, HandleContextMenuItemClickEvents));
			}
			(contextMenu.Items[1] as ToolStripMenuItem).Checked = scrollingEnabled;
			(contextMenu.Items[2] as ToolStripMenuItem).Checked = zoomingEnabled;
			this.ContextMenuStrip = contextMenu;
		}
		
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public Axis AxisX {
			get { return axisX; }
		}
		
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public Axis AxisY {
			get { return axisY; }
		}
		
		public bool CheckRangeOnPainting {
			get { return checkRangeOnPainting; }
			set { checkRangeOnPainting = value; }
		}
				
		public CurveList Curves {
			get { return curves; }
		}
		
		public bool EnforceTickBounds {
			get { return enforceTickBounds; }
			set { enforceTickBounds = value; }
		}
		
		
		public Font ScaleFont {
			get { return scaleFont; }
			set { scaleFont = value; }
		}
		
		public Pen AxisPen {
			get { return axisPen; }
			set { axisPen = value; }
		}
		
		public Pen TickPen {
			get { return tickPen; }
			set { tickPen = value; }
		}
		
		public Pen GridPen {
			get { return gridPen; }
			set { gridPen = value; }
		}
		
		public Brush ScaleBrush {
			get { return scaleBrush; }
			set { scaleBrush = value; }
		}
		
		public bool ScrollingEnabled {
			get { return scrollingEnabled; }
			set { scrollingEnabled = value; }
		}
		
		public SmoothingMode LineSmoothingMode {
			get { return lineSmoothingMode; }
			set { lineSmoothingMode = value; }
		}
		
		public SmoothingMode SymbolSmoothingMode {
			get { return symbolSmoothingMode; }
			set { symbolSmoothingMode = value; }
		}
		
		public TextRenderingHint TextRenderingHint {
			get { return textRenderingHint; }
			set { textRenderingHint = value; }
		}
		
		public bool ZoomingEnabled {
			get { return zoomingEnabled; }
			set { zoomingEnabled = value; }
		}
		
		public Point<double> GetDataPoint(Point screenPoint)
		{
			// points[i].X = (int)(x0 + dx * (dataPoints[i].X- xMin));
			// points[i].Y = (int)(y0 - dy * (dataPoints[i].Y - yMin));
			double x;
			double y;
			
			x = xMin + (screenPoint.X - x0) / dx;
			y = yMin + (y0 - screenPoint.Y) / dy;
			
			return new Point<double>(x, y);
		}
		
		protected Rectangle GetInvalidateRectangle(Rectangle contentRectangle, float lineWidth)
		{
			int w = (int)Math.Round(lineWidth, MidpointRounding.AwayFromZero);
			return new Rectangle(contentRectangle.X, contentRectangle.Y, contentRectangle.Width + w, contentRectangle.Height + w);
		}
		
		protected bool GetZoomRectangles()
		{
			int xMin;
			int yMin;
			int xMax;
			int yMax;
			int width;
			int height;
			bool shrink;
			
			previousZoomRectangle = zoomRectangle;
			
			if (mouseDownPoint.HasValue && mouseUpPoint.HasValue) {
				xMin = Math.Min(mouseDownPoint.Value.X, mouseUpPoint.Value.X);
				yMin = Math.Min(mouseDownPoint.Value.Y, mouseUpPoint.Value.Y);
				xMax = Math.Max(mouseDownPoint.Value.X, mouseUpPoint.Value.X);
				yMax = Math.Max(mouseDownPoint.Value.Y, mouseUpPoint.Value.Y);
				width = xMax - xMin;
				height = yMax - yMin;
				zoomRectangle = new Nullable<Rectangle>(new Rectangle(xMin, yMin, width, height));
			} else {
				zoomRectangle = null;
			}
			
			if (previousZoomRectangle.HasValue && zoomRectangle.HasValue) {
				xMin = Math.Min(previousZoomRectangle.Value.X, zoomRectangle.Value.X);
				yMin = Math.Min(previousZoomRectangle.Value.Y, zoomRectangle.Value.Y);
				xMax = Math.Max(previousZoomRectangle.Value.Right, zoomRectangle.Value.Right);
				yMax = Math.Max(previousZoomRectangle.Value.Bottom, zoomRectangle.Value.Bottom);
				width = xMax - xMin + 1;
				height = yMax - yMin + 1;
				zoomInvalidateRectangle = new Nullable<Rectangle>(new Rectangle(xMin, yMin, width, height));
				shrink = (zoomRectangle.Value.Left < previousZoomRectangle.Value.Left) || (zoomRectangle.Value.Right < previousZoomRectangle.Value.Right) ||
					(zoomRectangle.Value.Bottom < previousZoomRectangle.Value.Bottom) || (zoomRectangle.Value.Top < previousZoomRectangle.Value.Top);
			} else if (previousZoomRectangle.HasValue) {
				zoomInvalidateRectangle = previousZoomRectangle;
				shrink = true;
			} else if (zoomRectangle.HasValue) {
				zoomInvalidateRectangle = zoomRectangle;
				shrink = false;
			} else {
				zoomInvalidateRectangle = null;
				shrink = true;
			}
			
			return shrink;
		}
		
		public void SaveGnuPlotScript(string path)
		{
			using (GnuPlotTextWriter writer = new GnuPlotTextWriter(path, Terminal.EpsColour)) {
				int separatorLineWidth = 15;
				writer.OutputPath = System.IO.Path.GetFileNameWithoutExtension(path) + ".eps";
				writer.WriteLine("reset");
				writer.WriteLine("set encoding iso_8859_2");
				writer.WriteTerminal();
				writer.WriteOutput(true);
				writer.WriteSeparatorLine(separatorLineWidth);
				writer.WriteLine("set decimalsign \"{0}\"", System.Threading.Thread.CurrentThread.CurrentUICulture.NumberFormat.NumberDecimalSeparator);
				writer.WriteLine("set border 15");
				writer.WriteLine("set grid xtics linetype 0 linewidth 1");
				writer.WriteLine("set grid ytics linetype 0 linewidth 1");
				writer.WriteSeparatorLine(separatorLineWidth);
				writer.AxisX.Format = TetheredSun.GnuPlot.Axis.ToGnuPlotFormat("g");
				writer.AxisX.Range = new Interval(
					axisX.Autoscaled ? Double.NegativeInfinity : axisX.Minimum,
					axisX.Autoscaled ? Double.PositiveInfinity : axisX.Maximum);
				writer.AxisX.Label = axisX.Title.Text;
				writer.WriteAxisX();
				writer.WriteSeparatorLine(separatorLineWidth);
				writer.AxisY.Format = TetheredSun.GnuPlot.Axis.ToGnuPlotFormat("g");
				writer.AxisY.Range = new Interval(
					axisY.Autoscaled ? Double.NegativeInfinity : axisY.Minimum,
					axisY.Autoscaled ? Double.PositiveInfinity : axisY.Maximum);
				writer.AxisY.Label = axisY.Title.Text;
				writer.WriteAxisY();
				writer.WriteSeparatorLine(separatorLineWidth);
				writer.WriteCurveList(curves);
				writer.WriteSeparatorLine(separatorLineWidth);
				writer.WriteOutput(false);
			}
		}
		
		public void SaveImage(string path, ImageFormat format)
		{
			Bitmap image = new Bitmap(this.ClientRectangle.Width, this.ClientRectangle.Height, PixelFormat.Format32bppArgb);
			
			this.DrawToBitmap(image, this.ClientRectangle);
			image.Save(path, format);
		}
		
		protected bool Contains(Rectangle rectangle, Point point)
		{
			bool contains = (point.X >= rectangle.Left);
			
			contains &= (point.X <= rectangle.Right);
			contains &= (point.Y >= rectangle.Top);
			contains &= (point.Y <= rectangle.Bottom);
			
			return contains;
		}
		
		protected void DrawCursorPosition(Graphics graphics)
		{
			SizeF textSize;
			Point cursorPosition = this.PointToClient(Cursor.Position);
			
			if (Contains(this.ClientRectangle, Cursor.Position)) {
				string coordinateString;
				
				cursorCoordinates = GetDataPoint(cursorPosition);
				
				coordinateString = String.Format("({0:F2}; {1:F2})", cursorCoordinates.X, cursorCoordinates.Y);
				textSize = graphics.MeasureString(coordinateString, scaleFont);
				labelRectangle = new RectangleF(
					cursorPosition.X,
					cursorPosition.Y - textSize.Height,
					textSize.Width,
					textSize.Height
				);
				graphics.DrawEllipse(axisPen, cursorPosition.X, cursorPosition.Y, 3.0f, 3.0f);
				graphics.DrawString(coordinateString, scaleFont, scaleBrush, labelRectangle);
			}
		}
		
		protected void DrawFrame(Graphics graphics)
		{
			int				xc;
			int 			yc;
			int				temp;
			int				tickLength;
			int				xTickLabelSize;
			int				yTickLabelSize;
			int				bottomCorrection;
			int				leftCorrection;
			double			xTickMinimum;
			double			xTickMaximum;
			double			xTickStep;
			double			yTickMinimum;
			double			yTickMaximum;
			double			yTickStep;
			string			s;
			SizeF			textSize;

			if (xMin == xMax) {
				double x = xMax;
				double p = 0.1;
				if (x > 1E-100) {
					xMax = (1 + p) * x;
					xMin = (1 - p) * x;
				} else if (x < -1E-100) {
					xMax = (1 - p) * x;
					xMin = (1 + p) * x;
				} else {
					xMax = 1;
					xMin = -1;
				}
			}
			if (yMin == yMax) {
				double y = yMax;
				double p = 0.1;
				if (y > 1E-100) {
					yMax = (1 + p) * y;
					yMin = (1 - p) * y;
				} else if (y < -1E-100) {
					yMax = (1 - p) * y;
					yMin = (1 + p) * y;
				} else {
					yMax = 1;
					yMin = -1;
				}
			}
			yTickLabelSize = TickLabelSize(graphics, AxisType.Y, 3, ref yMin, ref yMax, out yTickMinimum, out yTickMaximum, out yTickStep);
			xTickLabelSize = TickLabelSize(graphics, AxisType.X, 3, ref xMin, ref xMax, out xTickMinimum, out xTickMaximum, out xTickStep) + 6;
			textSize = graphics.MeasureString("ABC", scaleFont);
			
			bottomCorrection = !axisX.Title.IsEmpty ? (int)(axisX.Title.Measure(graphics).Height * 1.2) : 0;
			leftCorrection = !axisY.Title.IsEmpty ?  (int)(axisY.Title.Measure(graphics).Height * 1.2) : 0;
			
			scaledAxisFrameRectangle = new Rectangle(
				drawRectangle.Left + leftCorrection,
				drawRectangle.Top,
				drawRectangle.Width - leftCorrection,
				drawRectangle.Height - bottomCorrection
			);
			
			axisFrameRectangle = new Rectangle(
				scaledAxisFrameRectangle.Left + yTickLabelSize,
				scaledAxisFrameRectangle.Top + 1,
				scaledAxisFrameRectangle.Width - yTickLabelSize,
				scaledAxisFrameRectangle.Height - 1 - xTickLabelSize
			);
			
			curveRectangle = new Rectangle(
				axisFrameRectangle.Left + drawRectangle.Width / 40,
				axisFrameRectangle.Top + drawRectangle.Height / 40,
				axisFrameRectangle.Width - drawRectangle.Width / 20,
				axisFrameRectangle.Height - drawRectangle.Height / 20
			);
			
			s = String.Format("{0:g}", xTickMaximum);
			textSize = graphics.MeasureString(s, scaleFont);
			temp = drawRectangle.Right - (int)Math.Floor(0.5 + (curveRectangle.Right - curveRectangle.Left) / (xMax-xMin) * (xTickMaximum - xMin)) - curveRectangle.Left - (int)textSize.Width / 2 - 1;
			if (temp < 0) curveRectangle.Width += temp;
			if ((curveRectangle.Width < 10) || (curveRectangle.Height < 10)) return;
			dx = curveRectangle.Width / (xMax - xMin);
			dy = curveRectangle.Height / (yMax - yMin);
			x0 = curveRectangle.Left;
			y0 = curveRectangle.Bottom;

			graphics.DrawLine(axisPen, axisFrameRectangle.Left, axisFrameRectangle.Top, axisFrameRectangle.Left, axisFrameRectangle.Bottom);
			graphics.DrawLine(axisPen, axisFrameRectangle.Left, axisFrameRectangle.Bottom, axisFrameRectangle.Right, axisFrameRectangle.Bottom);
			graphics.DrawLine(axisPen, axisFrameRectangle.Right, axisFrameRectangle.Bottom, axisFrameRectangle.Right, axisFrameRectangle.Top);
			graphics.DrawLine(axisPen, axisFrameRectangle.Right, axisFrameRectangle.Top, axisFrameRectangle.Left, axisFrameRectangle.Top);

			
			xAxisRectangle = new Rectangle(
				axisFrameRectangle.Left,
				axisFrameRectangle.Bottom,
				drawRectangle.Right - axisFrameRectangle.Left,
				drawRectangle.Bottom - axisFrameRectangle.Bottom
			);
			
			yAxisRectangle = new Rectangle(
				drawRectangle.Left,
				drawRectangle.Top,
				axisFrameRectangle.Left - drawRectangle.Left,
				axisFrameRectangle.Bottom - drawRectangle.Top
			);
			
			tickLength = Math.Max((drawRectangle.Right - drawRectangle.Left) / 100, 3);
			textSize = graphics.MeasureString("8", scaleFont);
			
			double xTickValue = xTickMinimum;
			xGridCount = 0;
			do {
				xGridCount++;
				xc = (int)Math.Floor(0.5 + x0 + dx * (xTickValue - xMin));
				// Draw grid:
				if (axisX.GridVisible) graphics.DrawLine(gridPen, xc, axisFrameRectangle.Bottom, xc, axisFrameRectangle.Top);
				// Draw ticks:
				graphics.DrawLine(tickPen, xc, axisFrameRectangle.Top, xc, axisFrameRectangle.Top + tickLength);
				graphics.DrawLine(tickPen, xc, axisFrameRectangle.Bottom - 1, xc, axisFrameRectangle.Bottom - 1 - tickLength);
				s = String.Format("{0:g}", (Math.Abs(xTickValue) < xTickStep / 4.0) ? 0.0 : xTickValue);
				textSize = graphics.MeasureString(s, scaleFont);
				labelRectangle = new RectangleF(
					xc - textSize.Width / 2,
					axisFrameRectangle.Bottom + scalePadding,
					textSize.Width,
					textSize.Height
				);
				graphics.DrawString(s, scaleFont, scaleBrush, labelRectangle);
			} while ((xTickValue += xTickStep) < xTickMaximum + xTickStep / 3);
			
			double yTickValue = yTickMinimum;
			yGridCount = 0;
			do {
				yGridCount++;
				yc = (int)Math.Floor(0.5 + y0 - dy * (yTickValue - yMin));
				if (axisY.GridVisible) graphics.DrawLine(gridPen, axisFrameRectangle.Left, yc, axisFrameRectangle.Right, yc);
				graphics.DrawLine(tickPen, axisFrameRectangle.Left, yc, axisFrameRectangle.Left + tickLength, yc);
				graphics.DrawLine(tickPen, axisFrameRectangle.Right - 1, yc, axisFrameRectangle.Right - 1 - tickLength, yc);
				s = String.Format("{0:g}", (Math.Abs(yTickValue) < yTickStep / 4.0) ? 0.0 : yTickValue);
				textSize = graphics.MeasureString(s, scaleFont);
				labelRectangle = new RectangleF(
					axisFrameRectangle.Left - scalePadding - textSize.Width,
					yc - textSize.Height / 2,
					textSize.Width,
					textSize.Height
				);
				graphics.DrawString(s, scaleFont, scaleBrush, labelRectangle);
			} while ((yTickValue += yTickStep) < yTickMaximum + yTickStep/3);
			
			if (!axisX.Title.IsEmpty) {
				titleRectangle = new Rectangle(
					axisFrameRectangle.Left,
					scaledAxisFrameRectangle.Bottom,
					drawRectangle.Right - axisFrameRectangle.Left,
					drawRectangle.Bottom - scaledAxisFrameRectangle.Bottom
				);
				
				axisX.Title.Draw(graphics, titleRectangle);
			}
			
			if (!axisY.Title.IsEmpty) {
				textSize = axisY.Title.Measure(graphics);
				state = graphics.Save();
				graphics.TranslateTransform(drawRectangle.Left, drawRectangle.Height / 2);
				graphics.RotateTransform(-90);
				axisY.Title.Draw(graphics, 0, textSize.Height / 2.0f);
				graphics.Restore(state);
			}
		}
		
		protected void DrawZoomRectangle(Graphics graphics)
		{
			if (!zoomRectangle.HasValue) return;
			
			graphics.FillRectangle(zoomBrush, zoomRectangle.Value);
			graphics.DrawRectangle(zoomPen, zoomRectangle.Value);
		}
		
		protected void HandleContextMenuItemClickEvents(object sender, EventArgs e)
		{
			ToolStripMenuItem menuItem = sender as ToolStripMenuItem;
			
			if (menuItem == null) return;
			
			if (menuItem.Text == contextMenuLabels[0]) {
				axisX.Autoscaled = true;
				axisY.Autoscaled = true;
				this.Refresh();
			} else if (menuItem.Text == contextMenuLabels[1]) {
				menuItem.Checked = !menuItem.Checked;
				scrollingEnabled = menuItem.Checked;
			} else if (menuItem.Text == contextMenuLabels[2]) {
				menuItem.Checked = !menuItem.Checked;
				zoomingEnabled = menuItem.Checked;
			} else if (menuItem.Text == contextMenuLabels[3]) {
				if (curves == null) return;
				if (curves.Count < 1) return;
				using (SaveFileDialog dialogue = new SaveFileDialog()) {
					dialogue.Filter = "GnuPlot scripts | *.plt";
					if (dialogue.ShowDialog() == DialogResult.OK) {
						SaveGnuPlotScript(dialogue.FileName);
					}
				}
			} else if (menuItem.Text == contextMenuLabels[4]) {
				if (curves == null) return;
				if (curves.Count < 1) return;
				using (SaveFileDialog dialogue = new SaveFileDialog()) {
					dialogue.Filter = "JPG | *.jpg | PNG | *.png | BMP | *.bmp";
					if (dialogue.ShowDialog() == DialogResult.OK) {
						ImageFormat format;
						
						switch (dialogue.FilterIndex) {
							case 2:
								format = ImageFormat.Png;
								break;
							case 3:
								format = ImageFormat.Bmp;
								break;
							case 1:
							default:
								format = ImageFormat.Jpeg;
								break;
						}
						
						SaveImage(dialogue.FileName, format);
					}
				}
			}
		}
		
		protected override void OnClick(EventArgs e)
		{
			base.OnClick(e);
			this.Focus();
		}
		
		protected override void OnDoubleClick(EventArgs e)
		{
			base.OnDoubleClick(e);
			
			Axis axis;
			Point cursorPosition = this.PointToClient(Cursor.Position);
			
			if (Contains(xAxisRectangle, cursorPosition)) {
				axis = axisX;
			} else if (Contains(yAxisRectangle, cursorPosition)) {
				axis = axisY;
			} else {
				return;
			}
			
			using (AxisBoundsEditor editor = new AxisBoundsEditor(axis.Minimum, axis.Maximum, axis.Autoscaled)) {
				if (editor.ShowDialog() == DialogResult.OK) {
					axis.Maximum = editor.Maximum;
					axis.Minimum = editor.Minimum;
					axis.Autoscaled = editor.Autoscaled;
					this.Refresh();
				}
			}
		}
		
		protected override void OnKeyDown(KeyEventArgs e)
		{
			base.OnKeyDown(e);
			controlPressed = e.Control;
		}
		
		protected override void OnKeyUp(KeyEventArgs e)
		{
			base.OnKeyUp(e);
			// If Esc is pressed, undo all zoom and scroll:
			if (e.KeyData == Keys.Escape) {
				if (!this.Focused) return;
				axisX.Autoscaled = true;
				axisY.Autoscaled = true;
				this.Refresh();
			} else if (e.KeyData == Keys.K) {
				if (showCursorPosition) this.Refresh();
			} else if (e.KeyData == Keys.S) {
				showCursorPosition = !showCursorPosition;
				this.Refresh();
			}
			controlPressed = false;
		}
		
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown(e);
			if (Contains(axisFrameRectangle, e.Location)) {
				mouseDownPoint = e.Location;
			} else {
				mouseDownPoint = null;
			}
			
		}
		
		protected override void OnMouseMove(MouseEventArgs e)
		{
			base.OnMouseMove(e);
			if (zoomingEnabled) {
				if (e.Button != MouseButtons.Left) {
					return;
				}

				mouseUpPoint = e.Location;
				GetZoomRectangles();

				if (zoomInvalidateRectangle.HasValue) {
					this.Invalidate(GetInvalidateRectangle(zoomInvalidateRectangle.Value, zoomPen.Width));
				}
			}
		}
		
		protected override void OnMouseUp(MouseEventArgs e)
		{
			base.OnMouseUp(e);

			if (zoomingEnabled && (e.Button == MouseButtons.Left)) {
				Zoom();
				zoomRectangle = null;
				this.Refresh();
			}
			zoomRectangle = null;
			mouseDownPoint = null;
			mouseUpPoint = null;
			previousZoomRectangle = null;
		}
		
		protected override void OnMouseWheel(MouseEventArgs e)
		{
			base.OnMouseWheel(e);
			
			if (Contains(xAxisRectangle, e.Location)) {
				if (controlPressed) {
					if (!zoomingEnabled) return;
					axisX.Autoscaled = false;
					axisX.Zoom(e.Delta / 120);
					this.Refresh();
				} else if (xGridCount > 0) {
					if (!scrollingEnabled) return;
					axisX.Autoscaled = false;
					axisX.Scroll(e.Delta / 120, xGridCount);
					this.Refresh();
				}
			} else if (Contains(yAxisRectangle, e.Location)) {
				if (controlPressed) {
					if (!zoomingEnabled) return;
					axisY.Autoscaled = false;
					axisY.Zoom(e.Delta / 120);
					this.Refresh();
				} else if (yGridCount > 0) {
					if (!scrollingEnabled) return;
					axisY.Autoscaled = false;
					axisY.Scroll(e.Delta / 120, yGridCount);
					this.Refresh();
				}
			}
		}
		
		protected override void OnPaint(PaintEventArgs e)
		{
			e.Graphics.SmoothingMode = lineSmoothingMode;
			e.Graphics.TextRenderingHint = textRenderingHint;
			
			base.OnPaint(e);
			
			UpdateAxisBounds();
			
			if (boundsUndefined) return;
			
			drawRectangle = new Rectangle(
				this.ClientRectangle.Left + this.Padding.Left,
				this.ClientRectangle.Top + this.Padding.Top,
				this.ClientRectangle.Width - this.Padding.Left - this.Padding.Right,
				this.ClientRectangle.Height - this.Padding.Top - this.Padding.Bottom
			);

			DrawFrame(e.Graphics);
			
			Point[] points;
			IList<Point<double>> dataPoints;
			
			e.Graphics.IntersectClip(axisFrameRectangle);
			
			foreach (Curve curve in curves) {
				if (curve.Visible) {
					if (checkRangeOnPainting) {
						lock (curve.Points.SyncRoot) {
							dataPoints = curve.Points.GetPointsInRange(xMin, xMax);
						}
						
						points = new Point[dataPoints.Count];
						
						unchecked {
							for (int i = 0; i < points.Length; i++) {
								points[i].X = (int)(x0 + dx * (dataPoints[i].X- xMin));
								points[i].Y = (int)(y0 - dy * (dataPoints[i].Y -yMin));
								if (points[i].X < -1073741376) points[i].X = -1073741376;
								if (points[i].Y < -1073740288) points[i].Y = -1073740288;
							}
						}
					} else {
						lock (curve.Points.SyncRoot) {
							dataPoints = curve.Points;
							
							points = new Point[dataPoints.Count];
						
							unchecked {
								for (int i = 0; i < points.Length; i++) {
									points[i].X = (int)(x0 + dx * (dataPoints[i].X- xMin));
									points[i].Y = (int)(y0 - dy * (dataPoints[i].Y -yMin));
									if (points[i].X < -1073741376) points[i].X = -1073741376;
									if (points[i].Y < -1073740288) points[i].Y = -1073740288;
								}
							}
						}
					}
					
					if (dataPoints.Count < 1) continue;
					
					if (curve.ErrorBar != null) {
						float error;
						DataPoint dataPoint;
						
						for (int i = 0; i < points.Length; i++) {
							dataPoint = dataPoints[i] as DataPoint;
							if (dataPoint == null) continue;
							
							error = (float)(dy * dataPoint.Error);
							curve.ErrorBar.Draw(e.Graphics, points[i], error);
						}
					}
					
					if (curve.HasLine && points.Length > 1) e.Graphics.DrawLines(curve.Line, points);
					
					if (curve.Symbol != null) {
						e.Graphics.SmoothingMode = symbolSmoothingMode;
						foreach (Point point in points) {
							curve.Symbol.Draw(e.Graphics, point);
						}
						e.Graphics.SmoothingMode = lineSmoothingMode;
					}
				}
			}
			
			if (showCursorPosition) DrawCursorPosition(e.Graphics);
			if (zoomingEnabled) DrawZoomRectangle(e.Graphics);
			
			e.Graphics.ResetClip();
		}
		
		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);
			this.Refresh();
		}
		
		[System.ComponentModel.EditorBrowsableAttribute()]
		protected override void OnCreateControl()
		{
			base.OnCreateControl();
			if (this.Parent != null) this.BackColor = this.Parent.BackColor;
		}
		
		protected int TickLabelSize(Graphics graphics, AxisType axis, int desiredNumberOfTicks, ref double minimum, ref double maximum, out double tickMinimum, out double tickMaximum, out double tickStep)
		{
			string	text;
			int		tickLabelSize;
			double	d;
			double	tickValue;
			SizeF 	textSize;
			
			d = maximum - minimum;
			tickStep = Math.Pow(10.0, Math.Floor(Math.Log10(d / desiredNumberOfTicks)));  // Step size to power of 10, tick labels 0, 1, 2, 3, ...*10^x
			if (d / tickStep > 5 * desiredNumberOfTicks) tickStep *= 5; // Tick labels 0, 5, 10, 15, ...*10^x
			else if (d / tickStep > 2 * desiredNumberOfTicks) tickStep *= 2; // Tick labels 0, 2, 4, 6, ...*10^x
			tickMinimum = tickStep * Math.Floor(2.0 + minimum / tickStep);
			while (Math.Abs(minimum - tickMinimum) > tickStep / 1.9) tickMinimum -= tickStep;
			tickMaximum = tickMinimum;
			while (Math.Abs(maximum - tickMaximum) > tickStep / 1.9) tickMaximum += tickStep;
			if (enforceTickBounds) {	// Range will be readjusted yTickMaximum tick
				if (tickMinimum < minimum) minimum = tickMinimum;
				if (tickMaximum > maximum) maximum = tickMaximum;
			} else	{		// Otherwise
				if (tickMinimum < minimum) tickMinimum += tickStep;
				if (tickMaximum > maximum) tickMaximum -= tickStep;
			}

			tickLabelSize = 0;
			
			for (tickValue = tickMinimum; tickValue < tickMaximum + tickStep / 2; tickValue += tickStep) {
				text = String.Format("{0:g}", (Math.Abs(tickValue) < tickStep / 4.0) ? 0.0 : tickValue);
				textSize = graphics.MeasureString(text, scaleFont);
				tickLabelSize = Math.Max(tickLabelSize, (int)((axis == AxisType.Y) ? textSize.Width : textSize.Height));
			}
			
			return 4 + tickLabelSize;
		}
		
		protected void UpdateAxisBounds()
		{
			if (curves.Count < 1) {
				boundsUndefined = true;
				return;
			}
			
			if (curves[0].Points.Count < 1) {
				boundsUndefined = true;
				return;
			}
			
			double xMinimum;
			double xMaximum;
			double yMinimum;
			double yMaximum;
			
			boundsUndefined = false;
			/*lock (curves[0].Points.SyncRoot) {
				curves[0].Points.UpdateBounds(out xMinimum, out xMaximum, out yMinimum, out yMaximum);
			}
			// TODO: Include other curves as well.*/
			curves.UpdateBounds(out xMinimum, out xMaximum, out yMinimum, out yMaximum);
			
			if (axisX.Autoscaled) {
				xMin = xMinimum;
				xMax = xMaximum;
				axisX.Minimum = xMin;
				axisX.Maximum = xMax;
			} else {
				xMin = axisX.Minimum;
				xMax = axisX.Maximum;
			}
			
			if (axisY.Autoscaled) {
				yMin = yMinimum;
				yMax = yMaximum;
				axisY.Minimum = yMin;
				axisY.Maximum = yMax;
			} else {
				yMin = axisY.Minimum;
				yMax = axisY.Maximum;
			}
			
			boundsUndefined = Double.IsNaN(xMin) || Double.IsNaN(xMax) || Double.IsNaN(yMin) || Double.IsNaN(yMax) ||
				Double.IsInfinity(xMin) || Double.IsInfinity(xMax) || Double.IsInfinity(yMin) || Double.IsInfinity(yMax);
		}
		
		protected void Zoom()
		{
			if (!zoomingEnabled) return;
			if (!zoomRectangle.HasValue) return;
			if (boundsUndefined) return;
			
			Point topLeftScreen = new Point(zoomRectangle.Value.X, zoomRectangle.Value.Y);
			Point bottomRightScreen = new Point(zoomRectangle.Value.Right, zoomRectangle.Value.Bottom);
			Point<double> topLeft = GetDataPoint(topLeftScreen);
			Point<double> bottomRight = GetDataPoint(bottomRightScreen);
			
			axisX.Autoscaled = false;
			axisX.Minimum = topLeft.X;
			axisX.Maximum = bottomRight.X;
			
			axisY.Autoscaled = false;
			axisY.Minimum = bottomRight.Y;
			axisY.Maximum = topLeft.Y;
		}
	}
}