﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2010.10.17.
 * Idő: 12:02
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;

namespace TetheredSun.IO
{
	/// <summary>
	/// Description of IniSection.
	/// </summary>
	public class IniSection : List<IniParameter>
	{
		public static string LeftDelimiter = "[";
		public static string RightDelimiter = "]";
		//
		protected string name;
		
		public IniSection(int capacity, string name) : base(capacity)
		{
			this.name = name;
		}
		
		public IniSection(IList<IniParameter> list, string name) : base(list)
		{ 
			this.name = name;
		}
		
		public IniSection(string name) : base()
		{
			this.name = name;
		}
		
		public IniSection(int capacity) : this(capacity, null) { }
		
		public IniSection(IList<IniParameter> list) : this(list, null) { }
		
		public IniSection() : base()
		{ 
			this.name = null;
		}
		
		
		public string Name {
			get { return name; }
			set { name = value; }
		}
		
		public IniParameter this[string key] {
			get {
				for (int i = 0; i < this.Count; i++) {
					if (this[i].Key == key) return this[i];
				}
				
				return null;
			}
		}
		
		public override string ToString()
		{
			string text = String.IsNullOrEmpty(name) ?
				String.Empty :
				String.Format("{0}{1}{2}{3}", LeftDelimiter, name, RightDelimiter, Environment.NewLine);
			
			foreach (IniParameter parameter in this) {
				text += parameter.ToString() + Environment.NewLine;
			}
			
			return text;
		}
		
		public static bool TryParse(string text, out IniSection section, out int endLineIndex)
		{
			if (String.IsNullOrEmpty(text)) {
				endLineIndex = -1;
				section = null;
				return false;
			}
			
			if (text.StartsWith(IniParameter.CommentSymbol)) {
				endLineIndex = -1;
				section = null;
				return false;
			}
			
			string[] lines = text.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
			string name = null;
			string current;
			int startIndex = 0;
			int capacity = lines.Length;
			IniParameter parameter;
			
			if (lines.Length < 1) {
				endLineIndex = -1;
				section = null;
				return false;
			}
			
			current = lines[0].Trim();
			
			if (current.StartsWith(IniSection.LeftDelimiter)) {
				int rightDelimiterIndex = current.IndexOf(IniSection.RightDelimiter);
				
				startIndex = 1;
				capacity--;
				
				if (rightDelimiterIndex > 0) {
					name = current.Substring(1, rightDelimiterIndex - 1);
				} else {
					name = null;
				}
			}
			
			section = new IniSection(capacity, name);
			endLineIndex = startIndex;
			
			for (int i = startIndex; i < lines.Length; i++) {
				endLineIndex = i;
				current = lines[i].Trim();
				if (current.StartsWith(IniSection.LeftDelimiter)) break;
				if (String.IsNullOrEmpty(current)) continue;
				if (IniParameter.TryParse(current, out parameter)) section.Add(parameter);
			}
			
			if (section.Count == 0) {
				section = null;
				return false;
			} else {
				return true;
			}
		}
		
	}
}
